@@ -23,6 +23,11 @@
 #include <curl/curl.h>
 #include <string.h>
 
+char* http_netif = NULL;
+int http_ipproto = HTTP_IPPROTO_ANY;
+
+#define MEGA_NZ_API_PUBKEY_PIN "sha256//0W38e765pAfPqS3DqSVOrPsC4MEOvRBaXQ7nY1AJ47E="
+
 // curlver.h: this macro was added in May 14, 2015
 #ifndef CURL_AT_LEAST_VERSION
 #define CURL_VERSION_BITS(x, y, z) ((x) << 16 | (y) << 8 | z)
@@ -122,7 +127,18 @@ struct http *http_new(void)
 #else
 	curl_easy_setopt(h->curl, CURLOPT_ENCODING, "");
 #endif
-	curl_easy_setopt(h->curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
+
+	switch (http_ipproto) {
+		case HTTP_IPPROTO_V4:
+			curl_easy_setopt(h->curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
+			break;
+		case HTTP_IPPROTO_V6:
+			curl_easy_setopt(h->curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6);
+			break;
+	}
+
+	if (http_netif)
+		curl_easy_setopt(h->curl, CURLOPT_INTERFACE, http_netif);
 
 	if (mega_debug & MEGA_DEBUG_HTTP)
 		curl_easy_setopt(h->curl, CURLOPT_VERBOSE, 1L);
@@ -137,6 +153,21 @@ struct http *http_new(void)
 
 	curl_easy_setopt(h->curl, CURLOPT_BUFFERSIZE, 256 * 1024L);
 
+	curl_easy_setopt(h->curl, CURLOPT_SSL_SESSIONID_CACHE, 0L);
+
+#if CURL_AT_LEAST_VERSION(7, 44, 0)
+	const gchar* pkp_disable = g_getenv("MEGATOOLS_PKP_DISABLE");
+	if (pkp_disable == NULL || strcmp(pkp_disable, "1")) {
+		if (curl_easy_setopt(h->curl, CURLOPT_PINNEDPUBLICKEY, MEGA_NZ_API_PUBKEY_PIN) != CURLE_OK) {
+			g_printerr("ERROR: Failed to setup public key pinning, aborting! You probably need a newer cURL library.\n");
+			exit(1);
+		} else {
+			curl_easy_setopt(h->curl, CURLOPT_CAINFO, NULL);
+			curl_easy_setopt(h->curl, CURLOPT_SSL_VERIFYPEER, 0L);
+		}
+	}
+#endif
+
 	h->headers = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
 #if STEALTH_MODE
@@ -312,7 +343,7 @@ GString *http_post(struct http *h, const gchar *url, const gchar *body, gssize b
 	if (body) {
 		curl_easy_setopt(h->curl, CURLOPT_NOBODY, 0L);
 		curl_easy_setopt(h->curl, CURLOPT_POSTFIELDS, body);
-		curl_easy_setopt(h->curl, CURLOPT_POSTFIELDSIZE, body_len);
+		curl_easy_setopt(h->curl, CURLOPT_POSTFIELDSIZE, (long)body_len);
 	} else {
 		curl_easy_setopt(h->curl, CURLOPT_NOBODY, 1L);
 		curl_easy_setopt(h->curl, CURLOPT_POSTFIELDS, NULL);
