@@ -29,6 +29,7 @@
 
 enum {
 	MEGA_ERROR_NO_HANDLE,
+	MEGA_ERROR_EXISTS,
 	MEGA_ERROR_OTHER
 };
 
@@ -82,6 +83,7 @@ struct mega_share_key {
 
 struct mega_node {
 	gchar *name;
+	gchar *name_collate_key;
 	gchar *handle;
 	gchar *parent_handle;
 	gchar *user_handle;
@@ -142,14 +144,11 @@ void mega_session_watch_status(struct mega_session *s, mega_status_callback cb,
 void mega_session_enable_previews(struct mega_session *s, gboolean enable);
 
 // this has side effect of the current session being closed
-gboolean mega_session_open(struct mega_session *s, const gchar *un, const gchar *pw, const gchar *sid, GError **err);
+gboolean mega_session_open(struct mega_session *s, const gchar *un, const gchar *pw, gint max_age, gboolean *is_new_session, GError **err);
 void mega_session_close(struct mega_session *s);
 const gchar *mega_session_get_sid(struct mega_session *s);
 
 gboolean mega_session_save(struct mega_session *s, GError **err);
-// this has side effect of the current session being closed
-gboolean mega_session_load(struct mega_session *s, const gchar *un, const gchar *pw, gint max_age, gchar **last_sid,
-			   GError **err);
 
 gboolean mega_session_get_user(struct mega_session *s, GError **err);
 gboolean mega_session_refresh(struct mega_session *s, GError **err);
@@ -177,6 +176,7 @@ gboolean mega_session_download_data(struct mega_session *s, struct mega_download
 				    GError **err);
 void mega_download_data_free(struct mega_download_data_params *params);
 
+struct mega_node* mega_session_get_node_by_handle(struct mega_session *s, const gchar* handle);
 gboolean mega_node_is_writable(struct mega_session *s, struct mega_node *n);
 
 gboolean mega_node_is_container(struct mega_node *n);
