@@ -20,10 +20,10 @@
 #include <curl/curl.h>
 #include <openssl/err.h>
 #include <openssl/ssl.h>
-#include <locale.h>
 
 #include "config.h"
 #include "tools.h"
+#include "http.h"
 
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -59,6 +59,10 @@ static gint transfer_worker_count = 5;
 static gint cache_timout = 10 * 60;
 static gboolean opt_enable_previews = BOOLEAN_UNSET_BUT_TRUE;
 static gboolean opt_disable_resume;
+static gchar *opt_netif;
+static gchar *opt_ipproto;
+
+static gboolean tool_use_colors = FALSE;
 
 static gboolean opt_debug_callback(const gchar *option_name, const gchar *value, gpointer data, GError **error)
 {
@@ -145,6 +149,12 @@ static GOptionEntry network_options[] = {
 	{ "proxy", '\0',
                 0, G_OPTION_ARG_STRING, &opt_proxy,
                 "Proxy setup string", "PROXY" },
+	{ "netif", '\0',
+                0, G_OPTION_ARG_STRING, &opt_netif,
+                "Network interface or local IP address used for outgoing connections", "NAME" },
+	{ "ip-proto", '\0',
+                0, G_OPTION_ARG_STRING, &opt_ipproto,
+                "Which protocol to prefer when connecting to mega.nz (v4, v6, or any)", "PROTO" },
 	{ NULL }
 };
 
@@ -154,7 +164,6 @@ static void init_openssl_locking()
 	// OpenSSL >= 1.1.0-pre4 doesn't require specific callback setup
 }
 #else
-#if GLIB_CHECK_VERSION(2, 32, 0)
 
 static GMutex *openssl_mutexes = NULL;
 
@@ -187,78 +196,10 @@ static void init_openssl_locking()
 	CRYPTO_set_locking_callback(openssl_locking_callback);
 }
 
-#else
-
-static GMutex **openssl_mutexes = NULL;
-
-static void openssl_locking_callback(int mode, int type, const char *file, int line)
-{
-	if (mode & CRYPTO_LOCK)
-		g_mutex_lock(openssl_mutexes[type]);
-	else
-		g_mutex_unlock(openssl_mutexes[type]);
-}
-
-static unsigned long openssl_thread_id_callback()
-{
-	unsigned long ret;
-	ret = (unsigned long)g_thread_self();
-	return ret;
-}
-
-static void init_openssl_locking()
-{
-	gint i;
-
-	// initialize OpenSSL locking for multi-threaded operation
-	openssl_mutexes = g_new(GMutex *, CRYPTO_num_locks());
-	for (i = 0; i < CRYPTO_num_locks(); i++)
-		openssl_mutexes[i] = g_mutex_new();
-
-	SSL_library_init();
-	CRYPTO_set_id_callback(openssl_thread_id_callback);
-	CRYPTO_set_locking_callback(openssl_locking_callback);
-}
-
-#endif
-#endif
-
-#ifdef G_OS_WIN32
-static gchar *get_tools_dir(void)
-{
-	gchar *path = NULL;
-	wchar_t *wpath;
-	DWORD len = PATH_MAX;
-
-	HMODULE handle = GetModuleHandleW(NULL);
-
-	wpath = g_new0(wchar_t, len);
-	if (GetModuleFileNameW(handle, wpath, len) < len)
-		path = g_utf16_to_utf8(wpath, -1, NULL, NULL, NULL);
-	g_free(wpath);
-
-	if (path == NULL)
-		path = g_strdup("");
-
-	gchar *dir = g_path_get_dirname(path);
-	g_free(path);
-	return dir;
-}
 #endif
 
 static void init(void)
 {
-#if !GLIB_CHECK_VERSION(2, 32, 0)
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-#endif
-
-	setlocale(LC_ALL, "");
-
-#if !GLIB_CHECK_VERSION(2, 36, 0)
-	g_type_init();
-#endif
-
 	g_setenv("GSETTINGS_BACKEND", "memory", TRUE);
 
 #ifndef G_OS_WIN32
@@ -266,20 +207,6 @@ static void init(void)
 #endif
 
 	init_openssl_locking();
-
-#ifdef G_OS_WIN32
-	gchar *tools_dir = get_tools_dir();
-
-	gchar *tmp = g_build_filename(tools_dir, "gio", NULL);
-	g_setenv("GIO_EXTRA_MODULES", tmp, TRUE);
-	g_free(tmp);
-
-	gchar *certs = g_build_filename(tools_dir, "ca-certificates.crt", NULL);
-	g_setenv("CA_CERT_PATH", certs, TRUE);
-	g_free(certs);
-
-	g_free(tools_dir);
-#endif
 }
 
 gboolean tool_is_stdout_tty(void)
@@ -334,11 +261,11 @@ void tool_show_progress(const gchar *file, const struct mega_status_data *data)
 		total_str = g_format_size_full(data->progress.total, G_FORMAT_SIZE_IEC_UNITS);
 		rate_str = g_format_size_full(rate, G_FORMAT_SIZE_IEC_UNITS);
 
-		if (tool_is_stdout_tty()) {
+		if (tool_use_colors) {
 			g_print(ESC_WHITE "%s" ESC_NORMAL ": "
 				ESC_YELLOW "%.2f%%" ESC_NORMAL " - "
 				"done " ESC_GREEN "%s" ESC_NORMAL
-				" (avg. %s/s)" ESC_CLREOL "\r",
+				" (avg. %s/s)\n",
 				file, percentage, total_str, rate_str);
 		} else {
 			g_print("%s: %.2f%% - done %s (avg. %s/s)\n", file, percentage, total_str, rate_str);
@@ -350,15 +277,16 @@ void tool_show_progress(const gchar *file, const struct mega_status_data *data)
 		done_str = g_format_size_full(now_done, G_FORMAT_SIZE_IEC_UNITS | G_FORMAT_SIZE_LONG_FORMAT);
 		total_str = g_format_size_full(data->progress.total, G_FORMAT_SIZE_IEC_UNITS);
 
-		if (tool_is_stdout_tty()) {
+		if (tool_use_colors) {
 			g_print(ESC_WHITE "%s" ESC_NORMAL ": "
 				ESC_YELLOW "%.2f%%" ESC_NORMAL " - "
-				ESC_GREEN "%s" ESC_BLUE " of %s" ESC_NORMAL
-				ESC_CLREOL "\r",
+				ESC_GREEN "%s" ESC_BLUE " of %s" ESC_NORMAL,
 				file, percentage, done_str, total_str);
 		} else {
-			g_print("%s: %.2f%% - %s of %s\n", file, percentage, done_str, total_str);
+			g_print("%s: %.2f%% - %s of %s", file, percentage, done_str, total_str);
 		}
+
+		g_print("%s", tool_is_stdout_tty() ? ESC_CLREOL "\r" : "\n");
 	} else {
 		// regular update
 		rate = (gdouble)size_diff * 1e6 / time_span;
@@ -368,15 +296,17 @@ void tool_show_progress(const gchar *file, const struct mega_status_data *data)
 		total_str = g_format_size_full(data->progress.total, G_FORMAT_SIZE_IEC_UNITS);
 		rate_str = g_format_size_full(rate, G_FORMAT_SIZE_IEC_UNITS);
 
-		if (tool_is_stdout_tty()) {
+		if (tool_use_colors) {
 			g_print(ESC_WHITE "%s" ESC_NORMAL ": "
 				ESC_YELLOW "%.2f%%" ESC_NORMAL " - "
 				ESC_GREEN "%s" ESC_BLUE " of %s" ESC_NORMAL
-				" (%s/s)" ESC_CLREOL "\r",
+				" (%s/s)",
 				file, percentage, done_str, total_str, rate_str);
 		} else {
-			g_print("%s: %.2f%% - %s of %s (%s/s)\n", file, percentage, done_str, total_str, rate_str);
+			g_print("%s: %.2f%% - %s of %s (%s/s)", file, percentage, done_str, total_str, rate_str);
 		}
+
+		g_print("%s", tool_is_stdout_tty() ? ESC_CLREOL "\r" : "\n");
 	}
 
 	last_update = timenow;
@@ -461,44 +391,16 @@ static void print_version(void)
 {
 	if (opt_version) {
 		g_print("megatools " VERSION " - command line tools for Mega.nz\n\n");
-		g_print("Written by Ondřej Jirman <megous@megous.com>, 2013\n");
+		g_print("Written by Ondrej Jirman <megous@megous.com>, 2013-2022\n");
 		g_print("Go to http://megatools.megous.com for more information\n");
 		exit(0);
 	}
 }
 
-gchar *tool_convert_filename(const gchar *path, gboolean local)
-{
-	gchar *locale_path;
-
-#ifdef G_OS_WIN32
-	locale_path = g_locale_to_utf8(path, -1, NULL, NULL, NULL);
-#else
-	if (local)
-		locale_path = g_strdup(path);
-	else
-		locale_path = g_locale_to_utf8(path, -1, NULL, NULL, NULL);
-#endif
-
-	if (locale_path == NULL) {
-		g_printerr(
-			"ERROR: Invalid filename locale, can't convert file names specified on the command line to UTF-8.\n");
-		exit(1);
-	}
-
-	return locale_path;
-}
-
-static void fix_options(GOptionEntry* list)
-{
-	for (int i = 0; list[i].long_name; i++) {
-		list[i].flags |= G_OPTION_FLAG_IN_MAIN;
-	}
-}
-
 void tool_init(gint *ac, gchar ***av, const gchar *tool_name, GOptionEntry *tool_entries, ToolInitFlags flags)
 {
 	GError *local_err = NULL;
+	gchar** args = NULL;
 
 	init();
 
@@ -506,47 +408,26 @@ void tool_init(gint *ac, gchar ***av, const gchar *tool_name, GOptionEntry *tool
 	if (tool_entries)
 		g_option_context_add_main_entries(opt_context, tool_entries, NULL);
 
-	if (flags & TOOL_INIT_UPLOAD_OPTS) {
-		GOptionGroup *upload_group =
-			g_option_group_new("upload", "Upload Options:", "Show upload options", NULL, NULL);
-		fix_options(upload_options);
-		g_option_group_add_entries(upload_group, upload_options);
-		g_option_context_add_group(opt_context, upload_group);
-	}
+	if (flags & TOOL_INIT_UPLOAD_OPTS)
+		g_option_context_add_main_entries(opt_context, upload_options, NULL);
 
-	if (flags & TOOL_INIT_DOWNLOAD_OPTS) {
-		GOptionGroup *download_group =
-			g_option_group_new("download", "Dwonload Options:", "Show download options", NULL, NULL);
-		fix_options(download_options);
-		g_option_group_add_entries(download_group, download_options);
-		g_option_context_add_group(opt_context, download_group);
-	}
+	if (flags & TOOL_INIT_DOWNLOAD_OPTS)
+		g_option_context_add_main_entries(opt_context, download_options, NULL);
 
-	if (flags & (TOOL_INIT_AUTH | TOOL_INIT_AUTH_OPTIONAL)) {
-		GOptionGroup *auth_group = g_option_group_new(
-			"auth", "Authentication Options:", "Show authentication options", NULL, NULL);
-		fix_options(auth_options);
-		g_option_group_add_entries(auth_group, auth_options);
-		g_option_context_add_group(opt_context, auth_group);
-	}
+	if (flags & (TOOL_INIT_AUTH | TOOL_INIT_AUTH_OPTIONAL))
+		g_option_context_add_main_entries(opt_context, auth_options, NULL);
 
-	GOptionGroup *network_group =
-		g_option_group_new("network", "Network Options:", "Show network options", NULL, NULL);
-	fix_options(network_options);
-	g_option_group_add_entries(network_group, network_options);
-	g_option_context_add_group(opt_context, network_group);
+	g_option_context_add_main_entries(opt_context, network_options, NULL);
+	g_option_context_add_main_entries(opt_context, basic_options, NULL);
 
-	GOptionGroup *basic_group = g_option_group_new("basic", "Basic Options:", "Show basic options", NULL, NULL);
-	fix_options(basic_options);
-	g_option_group_add_entries(basic_group, basic_options);
-	g_option_context_add_group(opt_context, basic_group);
-
-	if (!g_option_context_parse(opt_context, ac, av, &local_err)) {
+	if (!g_option_context_parse_strv(opt_context, av, &local_err)) {
 		g_printerr("ERROR: Option parsing failed: %s\n", local_err->message);
 		g_clear_error(&local_err);
 		exit(1);
 	}
 
+	*ac = g_strv_length(*av);
+
 	print_version();
 
 	if (!opt_no_config || opt_config) {
@@ -640,6 +521,17 @@ void tool_init(gint *ac, gchar ***av, const gchar *tool_name, GOptionEntry *tool
 				else
 					g_clear_error(&local_err);
 			}
+
+			if (g_key_file_has_key(kf, "UI", "Colors", NULL) && tool_is_stdout_tty()) {
+				tool_use_colors = g_key_file_get_boolean(kf, "UI", "Colors", &local_err);
+				if (local_err) {
+					g_printerr(
+						"WARNING: Invalid value for UI.Colors set in the config file: %s\n",
+						local_err->message);
+					g_clear_error(&local_err);
+					tool_use_colors = FALSE;
+				}
+			}
 		}
 	}
 
@@ -655,6 +547,23 @@ void tool_init(gint *ac, gchar ***av, const gchar *tool_name, GOptionEntry *tool
 			proxy = opt_proxy;
 	}
 
+	if (opt_netif) {
+		http_netif = opt_netif;
+	}
+
+	if (opt_ipproto) {
+		if (!strcmp(opt_ipproto, "v4"))
+			http_ipproto = HTTP_IPPROTO_V4;
+		else if (!strcmp(opt_ipproto, "v6"))
+			http_ipproto = HTTP_IPPROTO_V6;
+		else if (!strcmp(opt_ipproto, "any"))
+			http_ipproto = HTTP_IPPROTO_ANY;
+		else {
+			g_printerr("ERROR: Invalid --ip-proto option.\n");
+			exit(1);
+		}
+	}
+
 	if (!(flags & TOOL_INIT_AUTH))
 		return;
 
@@ -675,8 +584,7 @@ void tool_init(gint *ac, gchar ***av, const gchar *tool_name, GOptionEntry *tool
 struct mega_session *tool_start_session(ToolSessionFlags flags)
 {
 	GError *local_err = NULL;
-	gchar *sid = NULL;
-	gboolean loaded = FALSE;
+	gboolean is_new_session;
 
 	struct mega_session *s = mega_session_new();
 
@@ -700,29 +608,16 @@ struct mega_session *tool_start_session(ToolSessionFlags flags)
 		goto err;
 	}
 
-	// try to load cached session data (they are valid for 10 minutes since last
-	// user_get or refresh)
-	if (!mega_session_load(s, opt_username, opt_password, cache_timout, &sid, &local_err)) {
-		g_clear_error(&local_err);
-
-		if (!mega_session_open(s, opt_username, opt_password, sid, &local_err)) {
-			g_printerr("ERROR: Can't login to mega.nz: %s\n", local_err->message);
-			goto err;
-		}
-
-		if (!(flags & TOOL_SESSION_AUTH_ONLY)) {
-			if (!mega_session_refresh(s, &local_err)) {
-				g_printerr("ERROR: Can't read filesystem info from mega.nz: %s\n", local_err->message);
-				goto err;
-			}
-
-			loaded = TRUE;
-		}
-
-		mega_session_save(s, NULL);
+	// open the session
+	if (!mega_session_open(s, opt_username, opt_password, cache_timout, &is_new_session, &local_err)) {
+		g_printerr("ERROR: Can't login to mega.nz: %s\n", local_err->message);
+		goto err;
 	}
 
-	if (!(flags & TOOL_SESSION_AUTH_ONLY) && opt_reload_files && !loaded) {
+	if (is_new_session)
+		mega_session_save(s, NULL);
+
+	if (!(flags & TOOL_SESSION_AUTH_ONLY) && (opt_reload_files || is_new_session)) {
 		if (!mega_session_refresh(s, &local_err)) {
 			g_printerr("ERROR: Can't read filesystem info from mega.nz: %s\n", local_err->message);
 			goto err;
@@ -734,13 +629,11 @@ struct mega_session *tool_start_session(ToolSessionFlags flags)
 	mega_session_enable_previews(s, !!opt_enable_previews);
 	mega_session_set_resume(s, !opt_disable_resume);
 
-	g_free(sid);
 	return s;
 
 err:
 	mega_session_free(s);
 	g_clear_error(&local_err);
-	g_free(sid);
 	return NULL;
 }
 
